@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2011 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,13 +13,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import os
 import time
 from random import random
 
+import swift.common.db
 from swift.account import server as account_server
 from swift.common.db import AccountBroker
-from swift.common.utils import get_logger, audit_location_generator
+from swift.common.utils import get_logger, audit_location_generator, \
+    TRUE_VALUES
 from swift.common.daemon import Daemon
 
 from eventlet import Timeout
@@ -37,6 +38,8 @@ class AccountAuditor(Daemon):
         self.interval = int(conf.get('interval', 1800))
         self.account_passes = 0
         self.account_failures = 0
+        swift.common.db.DB_PREALLOCATION = \
+            conf.get('db_preallocation', 't').lower() in TRUE_VALUES
 
     def run_forever(self, *args, **kwargs):
         """Run the account audit until stopped."""
