@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2011 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,9 +16,10 @@
 """
 Cloud Files client library used internally
 """
+
 import socket
 from urllib import quote as _quote
-from urlparse import urlparse, urlunparse
+from urlparse import urlparse, urlunparse, urljoin
 
 try:
     from eventlet.green.httplib import HTTPException, HTTPSConnection
@@ -53,16 +54,18 @@ def quote(value, safe='/'):
 try:
     # simplejson is popular and pretty good
     from simplejson import loads as json_loads
+    from simplejson import dumps as json_dumps
 except ImportError:
     # 2.6 will have a json module in the stdlib
     from json import loads as json_loads
+    from json import dumps as json_dumps
 
 
 class ClientException(Exception):
 
     def __init__(self, msg, http_scheme='', http_host='', http_port='',
                  http_path='', http_query='', http_status=0, http_reason='',
-                 http_device=''):
+                 http_device='', http_response_content=''):
         Exception.__init__(self, msg)
         self.msg = msg
         self.http_scheme = http_scheme
@@ -73,6 +76,7 @@ class ClientException(Exception):
         self.http_status = http_status
         self.http_reason = http_reason
         self.http_device = http_device
+        self.http_response_content = http_response_content
 
     def __str__(self):
         a = self.msg
@@ -102,6 +106,12 @@ class ClientException(Exception):
                 b = '%s: device %s' % (b, self.http_device)
             else:
                 b = 'device %s' % self.http_device
+        if self.http_response_content:
+            if len(self.http_response_content) <= 60:
+                b += '   %s' % self.http_response_content
+            else:
+                b += '  [first 60 chars of response] %s' % \
+                   self.http_response_content[:60]
         return b and '%s: %s' % (a, b) or a
 
 
@@ -129,23 +139,32 @@ def http_connection(url, proxy=None):
     return parsed, conn
 
 
-def get_auth(url, user, key, snet=False):
-    """
-    Get authentication/authorization credentials.
+def json_request(method, url, **kwargs):
+    """Takes a request in json parse it and return in json"""
+    kwargs.setdefault('headers', {})
+    if 'body' in kwargs:
+        kwargs['headers']['Content-Type'] = 'application/json'
+        kwargs['body'] = json_dumps(kwargs['body'])
+    parsed, conn = http_connection(url)
+    conn.request(method, parsed.path, **kwargs)
+    resp = conn.getresponse()
+    body = resp.read()
+    if body:
+        try:
+            body = json_loads(body)
+        except ValueError:
+            body = None
+    if not body or resp.status < 200 or resp.status >= 300:
+        raise ClientException('Auth GET failed', http_scheme=parsed.scheme,
+                              http_host=conn.host,
+                              http_port=conn.port,
+                              http_path=parsed.path,
+                              http_status=resp.status,
+                              http_reason=resp.reason)
+    return resp, body
 
-    The snet parameter is used for Rackspace's ServiceNet internal network
-    implementation. In this function, it simply adds *snet-* to the beginning
-    of the host name for the returned storage URL. With Rackspace Cloud Files,
-    use of this network path causes no bandwidth charges but requires the
-    client to be running on Rackspace's ServiceNet network.
 
-    :param url: authentication/authorization URL
-    :param user: user to authenticate as
-    :param key: key or password for authorization
-    :param snet: use SERVICENET internal network (see above), default is False
-    :returns: tuple of (storage URL, auth token)
-    :raises ClientException: HTTP GET request to auth URL failed
-    """
+def _get_auth_v1_0(url, user, key, snet):
     parsed, conn = http_connection(url)
     conn.request('GET', parsed.path, '',
                  {'X-Auth-User': user, 'X-Auth-Key': key})
@@ -160,12 +179,70 @@ def get_auth(url, user, key, snet=False):
     if snet:
         parsed = list(urlparse(url))
         # Second item in the list is the netloc
-        parsed[1] = 'snet-' + parsed[1]
+        netloc = parsed[1]
+        parsed[1] = 'snet-' + netloc
         url = urlunparse(parsed)
     return url, resp.getheader('x-storage-token',
                                                 resp.getheader('x-auth-token'))
 
 
+def _get_auth_v2_0(url, user, tenant_name, key, snet):
+    body = {'auth': {'passwordCredentials':
+                      {'password': key, 'username': user},
+                     'tenantName': tenant_name}}
+    token_url = urljoin(url, "tokens")
+    resp, body = json_request("POST", token_url, body=body)
+    token_id = None
+    try:
+        url = None
+        catalogs = body['access']['serviceCatalog']
+        for service in catalogs:
+            if service['type'] == 'object-store':
+                url = service['endpoints'][0]['publicURL']
+        token_id = body['access']['token']['id']
+        if not url:
+            raise ClientException("There is no object-store endpoint " \
+                                  "on this auth server.")
+    except(KeyError, IndexError):
+        raise ClientException("Error while getting answers from auth server")
+
+    if snet:
+        parsed = list(urlparse(url))
+        # Second item in the list is the netloc
+        parsed[1] = 'snet-' + parsed[1]
+        url = urlunparse(parsed)
+
+    return url, token_id
+
+
+def get_auth(url, user, key, snet=False, tenant_name=None, auth_version="1.0"):
+    """
+    Get authentication/authorization credentials.
+
+    The snet parameter is used for Rackspace's ServiceNet internal network
+    implementation. In this function, it simply adds *snet-* to the beginning
+    of the host name for the returned storage URL. With Rackspace Cloud Files,
+    use of this network path causes no bandwidth charges but requires the
+    client to be running on Rackspace's ServiceNet network.
+
+    :param url: authentication/authorization URL
+    :param user: user to authenticate as
+    :param key: key or password for authorization
+    :param snet: use SERVICENET internal network (see above), default is False
+    :param auth_version: OpenStack auth version, default is 1.0
+    :param tenant_name: The tenant/account name, required when connecting
+                        to a auth 2.0 system.
+    :returns: tuple of (storage URL, auth token)
+    :raises: ClientException: HTTP GET request to auth URL failed
+    """
+    if auth_version in ["1.0", "1"]:
+        return _get_auth_v1_0(url, user, key, snet)
+    elif auth_version in ["2.0", "2"]:
+        if not tenant_name:
+            raise ClientException('No tenant specified')
+        return _get_auth_v2_0(url, user, tenant_name, key, snet)
+
+
 def get_account(url, token, marker=None, limit=None, prefix=None,
                 http_conn=None, full_listing=False):
     """
@@ -211,11 +288,11 @@ def get_account(url, token, marker=None, limit=None, prefix=None,
     for header, value in resp.getheaders():
         resp_headers[header.lower()] = value
     if resp.status < 200 or resp.status >= 300:
-        resp.read()
+        body = resp.read()
         raise ClientException('Account GET failed', http_scheme=parsed.scheme,
                 http_host=conn.host, http_port=conn.port,
                 http_path=parsed.path, http_query=qs, http_status=resp.status,
-                http_reason=resp.reason)
+                http_reason=resp.reason, http_response_content=body)
     if resp.status == 204:
         resp.read()
         return resp_headers, []
@@ -240,12 +317,12 @@ def head_account(url, token, http_conn=None):
         parsed, conn = http_connection(url)
     conn.request('HEAD', parsed.path, '', {'X-Auth-Token': token})
     resp = conn.getresponse()
-    resp.read()
+    body = resp.read()
     if resp.status < 200 or resp.status >= 300:
         raise ClientException('Account HEAD failed', http_scheme=parsed.scheme,
                 http_host=conn.host, http_port=conn.port,
                 http_path=parsed.path, http_status=resp.status,
-                http_reason=resp.reason)
+                http_reason=resp.reason, http_response_content=body)
     resp_headers = {}
     for header, value in resp.getheaders():
         resp_headers[header.lower()] = value
@@ -270,12 +347,16 @@ def post_account(url, token, headers, http_conn=None):
     headers['X-Auth-Token'] = token
     conn.request('POST', parsed.path, '', headers)
     resp = conn.getresponse()
-    resp.read()
+    body = resp.read()
     if resp.status < 200 or resp.status >= 300:
         raise ClientException('Account POST failed',
-                http_scheme=parsed.scheme, http_host=conn.host,
-                http_port=conn.port, http_path=parsed.path,
-                http_status=resp.status, http_reason=resp.reason)
+                              http_scheme=parsed.scheme,
+                              http_host=conn.host,
+                              http_port=conn.port,
+                              http_path=parsed.path,
+                              http_status=resp.status,
+                              http_reason=resp.reason,
+                              http_response_content=body)
 
 
 def get_container(url, token, container, marker=None, limit=None,
@@ -329,11 +410,12 @@ def get_container(url, token, container, marker=None, limit=None,
     conn.request('GET', '%s?%s' % (path, qs), '', {'X-Auth-Token': token})
     resp = conn.getresponse()
     if resp.status < 200 or resp.status >= 300:
-        resp.read()
+        body = resp.read()
         raise ClientException('Container GET failed',
                 http_scheme=parsed.scheme, http_host=conn.host,
                 http_port=conn.port, http_path=path, http_query=qs,
-                http_status=resp.status, http_reason=resp.reason)
+                http_status=resp.status, http_reason=resp.reason,
+                http_response_content=body)
     resp_headers = {}
     for header, value in resp.getheaders():
         resp_headers[header.lower()] = value
@@ -366,12 +448,12 @@ def head_container(url, token, container, http_conn=None, headers=None):
         req_headers.update(headers)
     conn.request('HEAD', path, '', req_headers)
     resp = conn.getresponse()
-    resp.read()
+    body = resp.read()
     if resp.status < 200 or resp.status >= 300:
         raise ClientException('Container HEAD failed',
                 http_scheme=parsed.scheme, http_host=conn.host,
                 http_port=conn.port, http_path=path, http_status=resp.status,
-                http_reason=resp.reason)
+                http_reason=resp.reason, http_response_content=body)
     resp_headers = {}
     for header, value in resp.getheaders():
         resp_headers[header.lower()] = value
@@ -400,12 +482,12 @@ def put_container(url, token, container, headers=None, http_conn=None):
     headers['X-Auth-Token'] = token
     conn.request('PUT', path, '', headers)
     resp = conn.getresponse()
-    resp.read()
+    body = resp.read()
     if resp.status < 200 or resp.status >= 300:
         raise ClientException('Container PUT failed',
                 http_scheme=parsed.scheme, http_host=conn.host,
                 http_port=conn.port, http_path=path, http_status=resp.status,
-                http_reason=resp.reason)
+                http_reason=resp.reason, http_response_content=body)
 
 
 def post_container(url, token, container, headers, http_conn=None):
@@ -428,12 +510,12 @@ def post_container(url, token, container, headers, http_conn=None):
     headers['X-Auth-Token'] = token
     conn.request('POST', path, '', headers)
     resp = conn.getresponse()
-    resp.read()
+    body = resp.read()
     if resp.status < 200 or resp.status >= 300:
         raise ClientException('Container POST failed',
                 http_scheme=parsed.scheme, http_host=conn.host,
                 http_port=conn.port, http_path=path, http_status=resp.status,
-                http_reason=resp.reason)
+                http_reason=resp.reason, http_response_content=body)
 
 
 def delete_container(url, token, container, http_conn=None):
@@ -454,12 +536,12 @@ def delete_container(url, token, container, http_conn=None):
     path = '%s/%s' % (parsed.path, quote(container))
     conn.request('DELETE', path, '', {'X-Auth-Token': token})
     resp = conn.getresponse()
-    resp.read()
+    body = resp.read()
     if resp.status < 200 or resp.status >= 300:
         raise ClientException('Container DELETE failed',
                 http_scheme=parsed.scheme, http_host=conn.host,
                 http_port=conn.port, http_path=path, http_status=resp.status,
-                http_reason=resp.reason)
+                http_reason=resp.reason, http_response_content=body)
 
 
 def get_object(url, token, container, name, http_conn=None,
@@ -489,10 +571,11 @@ def get_object(url, token, container, name, http_conn=None,
     conn.request('GET', path, '', {'X-Auth-Token': token})
     resp = conn.getresponse()
     if resp.status < 200 or resp.status >= 300:
-        resp.read()
+        body = resp.read()
         raise ClientException('Object GET failed', http_scheme=parsed.scheme,
                 http_host=conn.host, http_port=conn.port, http_path=path,
-                http_status=resp.status, http_reason=resp.reason)
+                http_status=resp.status, http_reason=resp.reason,
+                http_response_content=body)
     if resp_chunk_size:
 
         def _object_body():
@@ -530,11 +613,12 @@ def head_object(url, token, container, name, http_conn=None):
     path = '%s/%s/%s' % (parsed.path, quote(container), quote(name))
     conn.request('HEAD', path, '', {'X-Auth-Token': token})
     resp = conn.getresponse()
-    resp.read()
+    body = resp.read()
     if resp.status < 200 or resp.status >= 300:
         raise ClientException('Object HEAD failed', http_scheme=parsed.scheme,
                 http_host=conn.host, http_port=conn.port, http_path=path,
-                http_status=resp.status, http_reason=resp.reason)
+                http_status=resp.status, http_reason=resp.reason,
+                http_response_content=body)
     resp_headers = {}
     for header, value in resp.getheaders():
         resp_headers[header.lower()] = value
@@ -624,11 +708,12 @@ def put_object(url, token=None, container=None, name=None, contents=None,
     else:
         conn.request('PUT', path, contents, headers)
     resp = conn.getresponse()
-    resp.read()
+    body = resp.read()
     if resp.status < 200 or resp.status >= 300:
         raise ClientException('Object PUT failed', http_scheme=parsed.scheme,
                 http_host=conn.host, http_port=conn.port, http_path=path,
-                http_status=resp.status, http_reason=resp.reason)
+                http_status=resp.status, http_reason=resp.reason,
+                http_response_content=body)
     return resp.getheader('etag', '').strip('"')
 
 
@@ -653,11 +738,12 @@ def post_object(url, token, container, name, headers, http_conn=None):
     headers['X-Auth-Token'] = token
     conn.request('POST', path, '', headers)
     resp = conn.getresponse()
-    resp.read()
+    body = resp.read()
     if resp.status < 200 or resp.status >= 300:
         raise ClientException('Object POST failed', http_scheme=parsed.scheme,
                 http_host=conn.host, http_port=conn.port, http_path=path,
-                http_status=resp.status, http_reason=resp.reason)
+                http_status=resp.status, http_reason=resp.reason,
+                http_response_content=body)
 
 
 def delete_object(url, token=None, container=None, name=None, http_conn=None,
@@ -695,19 +781,21 @@ def delete_object(url, token=None, container=None, name=None, http_conn=None,
         headers['X-Auth-Token'] = token
     conn.request('DELETE', path, '', headers)
     resp = conn.getresponse()
-    resp.read()
+    body = resp.read()
     if resp.status < 200 or resp.status >= 300:
         raise ClientException('Object DELETE failed',
                 http_scheme=parsed.scheme, http_host=conn.host,
                 http_port=conn.port, http_path=path, http_status=resp.status,
-                http_reason=resp.reason)
+                http_reason=resp.reason, http_response_content=body)
 
 
 class Connection(object):
     """Convenience class to make requests that will also retry the request"""
 
     def __init__(self, authurl, user, key, retries=5, preauthurl=None,
-                 preauthtoken=None, snet=False, starting_backoff=1):
+                 preauthtoken=None, snet=False, starting_backoff=1,
+                 tenant_name=None,
+                 auth_version="1"):
         """
         :param authurl: authentication URL
         :param user: user name to authenticate as
@@ -717,6 +805,9 @@ class Connection(object):
         :param preauthtoken: authentication token (if you have already
                              authenticated)
         :param snet: use SERVICENET internal network default is False
+        :param auth_version: OpenStack auth version, default is 1.0
+        :param tenant_name: The tenant/account name, required when connecting
+                            to a auth 2.0 system.
         """
         self.authurl = authurl
         self.user = user
@@ -728,9 +819,14 @@ class Connection(object):
         self.attempts = 0
         self.snet = snet
         self.starting_backoff = starting_backoff
+        self.auth_version = auth_version
+        self.tenant_name = tenant_name
 
     def get_auth(self):
-        return get_auth(self.authurl, self.user, self.key, snet=self.snet)
+        return get_auth(self.authurl, self.user,
+                        self.key, snet=self.snet,
+                        tenant_name=self.tenant_name,
+                        auth_version=self.auth_version)
 
     def http_connection(self):
         return http_connection(self.url)
