@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2011 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -76,3 +76,17 @@ class EmptyRingError(RingBuilderError):
 
 class DuplicateDeviceError(RingBuilderError):
     pass
+
+
+class ListingIterError(Exception):
+    pass
+
+
+class ListingIterNotFound(ListingIterError):
+    pass
+
+
+class ListingIterNotAuthorized(ListingIterError):
+
+    def __init__(self, aresp):
+        self.aresp = aresp
