@@ -98,7 +98,6 @@ class TempAuth(object):
                     url += ip
                     url += ':' + conf.get('bind_port', '8080') + '/v1/' + \
                            self.reseller_prefix + conf_key.split('_')[1]
-                groups = values
                 self.users[conf_key.split('_', 1)[1].replace('_', ':')] = {
                     'key': key, 'url': url, 'groups': values}
 
@@ -159,7 +158,7 @@ class TempAuth(object):
                     version, rest = split_path(env.get('PATH_INFO', ''),
                                                1, 2, True)
                 except ValueError:
-                    return HTTPNotFound()(env, start_response)
+                    version, rest = None, None
                 if rest and rest.startswith(self.reseller_prefix):
                     # Handle anonymous access to accounts I'm the definitive
                     # auth for.
