@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2011 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -29,14 +29,18 @@ from webob.exc import HTTPAccepted, HTTPBadRequest, HTTPConflict, \
     HTTPCreated, HTTPInternalServerError, HTTPNoContent, \
     HTTPNotFound, HTTPPreconditionFailed, HTTPMethodNotAllowed
 
+import swift.common.db
 from swift.common.db import ContainerBroker
 from swift.common.utils import get_logger, get_param, hash_path, \
-    normalize_timestamp, storage_directory, split_path, validate_sync_to
+    normalize_timestamp, storage_directory, split_path, validate_sync_to, \
+    TRUE_VALUES
 from swift.common.constraints import CONTAINER_LISTING_LIMIT, \
     check_mount, check_float, check_utf8
 from swift.common.bufferedhttp import http_connect
 from swift.common.exceptions import ConnectionTimeout
 from swift.common.db_replicator import ReplicatorRpc
+from swift.common.http import HTTP_NOT_FOUND, is_success, \
+    HTTPInsufficientStorage
 
 DATADIR = 'containers'
 
@@ -52,7 +56,7 @@ class ContainerController(object):
         self.logger = get_logger(conf, log_route='container-server')
         self.root = conf.get('devices', '/srv/node/')
         self.mount_check = conf.get('mount_check', 'true').lower() in \
-                              ('true', 't', '1', 'on', 'yes', 'y')
+                              TRUE_VALUES
         self.node_timeout = int(conf.get('node_timeout', 3))
         self.conn_timeout = float(conf.get('conn_timeout', 0.5))
         self.allowed_sync_hosts = [h.strip()
@@ -62,6 +66,10 @@ class ContainerController(object):
             ContainerBroker, self.mount_check, logger=self.logger)
         self.auto_create_account_prefix = \
             conf.get('auto_create_account_prefix') or '.'
+        if conf.get('allow_versions', 'f').lower() in TRUE_VALUES:
+            self.save_headers.append('x-versions-location')
+        swift.common.db.DB_PREALLOCATION = \
+            conf.get('db_preallocation', 't').lower() in TRUE_VALUES
 
     def _get_container_broker(self, drive, part, account, container):
         """
@@ -113,10 +121,9 @@ class ContainerController(object):
                 with Timeout(self.node_timeout):
                     account_response = conn.getresponse()
                     account_response.read()
-                    if account_response.status == 404:
+                    if account_response.status == HTTP_NOT_FOUND:
                         return HTTPNotFound(request=req)
-                    elif account_response.status < 200 or \
-                            account_response.status > 299:
+                    elif not is_success(account_response.status):
                         self.logger.error(_('ERROR Account update failed '
                             'with %(ip)s:%(port)s/%(device)s (will retry '
                             'later): Response %(status)s %(reason)s'),
@@ -133,38 +140,47 @@ class ContainerController(object):
 
     def DELETE(self, req):
         """Handle HTTP DELETE request."""
+        start_time = time.time()
         try:
             drive, part, account, container, obj = split_path(
                 unquote(req.path), 4, 5, True)
         except ValueError, err:
+            self.logger.increment('DELETE.errors')
             return HTTPBadRequest(body=str(err), content_type='text/plain',
                                 request=req)
         if 'x-timestamp' not in req.headers or \
                     not check_float(req.headers['x-timestamp']):
+            self.logger.increment('DELETE.errors')
             return HTTPBadRequest(body='Missing timestamp', request=req,
                         content_type='text/plain')
         if self.mount_check and not check_mount(self.root, drive):
-            return Response(status='507 %s is not mounted' % drive)
+            self.logger.increment('DELETE.errors')
+            return HTTPInsufficientStorage(drive=drive, request=req)
         broker = self._get_container_broker(drive, part, account, container)
         if account.startswith(self.auto_create_account_prefix) and obj and \
                 not os.path.exists(broker.db_file):
             broker.initialize(normalize_timestamp(
                 req.headers.get('x-timestamp') or time.time()))
         if not os.path.exists(broker.db_file):
+            self.logger.timing_since('DELETE.timing', start_time)
             return HTTPNotFound()
         if obj:     # delete object
             broker.delete_object(obj, req.headers.get('x-timestamp'))
+            self.logger.timing_since('DELETE.timing', start_time)
             return HTTPNoContent(request=req)
         else:
             # delete container
             if not broker.empty():
+                self.logger.increment('DELETE.errors')
                 return HTTPConflict(request=req)
             existed = float(broker.get_info()['put_timestamp']) and \
                       not broker.is_deleted()
             broker.delete_db(req.headers['X-Timestamp'])
             if not broker.is_deleted():
+                self.logger.increment('DELETE.errors')
                 return HTTPConflict(request=req)
             resp = self.account_update(req, account, container, broker)
+            self.logger.timing_since('DELETE.timing', start_time)
             if resp:
                 return resp
             if existed:
@@ -173,23 +189,28 @@ class ContainerController(object):
 
     def PUT(self, req):
         """Handle HTTP PUT request."""
+        start_time = time.time()
         try:
             drive, part, account, container, obj = split_path(
                 unquote(req.path), 4, 5, True)
         except ValueError, err:
+            self.logger.increment('PUT.errors')
             return HTTPBadRequest(body=str(err), content_type='text/plain',
                                 request=req)
         if 'x-timestamp' not in req.headers or \
                     not check_float(req.headers['x-timestamp']):
+            self.logger.increment('PUT.errors')
             return HTTPBadRequest(body='Missing timestamp', request=req,
                         content_type='text/plain')
         if 'x-container-sync-to' in req.headers:
             err = validate_sync_to(req.headers['x-container-sync-to'],
                                    self.allowed_sync_hosts)
             if err:
+                self.logger.increment('PUT.errors')
                 return HTTPBadRequest(err)
         if self.mount_check and not check_mount(self.root, drive):
-            return Response(status='507 %s is not mounted' % drive)
+            self.logger.increment('PUT.errors')
+            return HTTPInsufficientStorage(drive=drive, request=req)
         timestamp = normalize_timestamp(req.headers['x-timestamp'])
         broker = self._get_container_broker(drive, part, account, container)
         if obj:     # put container object
@@ -197,9 +218,11 @@ class ContainerController(object):
                     not os.path.exists(broker.db_file):
                 broker.initialize(timestamp)
             if not os.path.exists(broker.db_file):
+                self.logger.timing_since('PUT.timing', start_time)
                 return HTTPNotFound()
             broker.put_object(obj, timestamp, int(req.headers['x-size']),
                 req.headers['x-content-type'], req.headers['x-etag'])
+            self.logger.timing_since('PUT.timing', start_time)
             return HTTPCreated(request=req)
         else:   # put container
             if not os.path.exists(broker.db_file):
@@ -209,6 +232,7 @@ class ContainerController(object):
                 created = broker.is_deleted()
                 broker.update_put_timestamp(timestamp)
                 if broker.is_deleted():
+                    self.logger.increment('PUT.errors')
                     return HTTPConflict(request=req)
             metadata = {}
             metadata.update((key, (value, timestamp))
@@ -223,6 +247,7 @@ class ContainerController(object):
                         broker.set_x_container_sync_points(-1, -1)
                 broker.update_metadata(metadata)
             resp = self.account_update(req, account, container, broker)
+            self.logger.timing_since('PUT.timing', start_time)
             if resp:
                 return resp
             if created:
@@ -232,18 +257,22 @@ class ContainerController(object):
 
     def HEAD(self, req):
         """Handle HTTP HEAD request."""
+        start_time = time.time()
         try:
             drive, part, account, container, obj = split_path(
                 unquote(req.path), 4, 5, True)
         except ValueError, err:
+            self.logger.increment('HEAD.errors')
             return HTTPBadRequest(body=str(err), content_type='text/plain',
                                 request=req)
         if self.mount_check and not check_mount(self.root, drive):
-            return Response(status='507 %s is not mounted' % drive)
+            self.logger.increment('HEAD.errors')
+            return HTTPInsufficientStorage(drive=drive, request=req)
         broker = self._get_container_broker(drive, part, account, container)
         broker.pending_timeout = 0.1
         broker.stale_reads_ok = True
         if broker.is_deleted():
+            self.logger.timing_since('HEAD.timing', start_time)
             return HTTPNotFound(request=req)
         info = broker.get_info()
         headers = {
@@ -256,22 +285,27 @@ class ContainerController(object):
             for key, (value, timestamp) in broker.metadata.iteritems()
             if value != '' and (key.lower() in self.save_headers or
                                 key.lower().startswith('x-container-meta-')))
+        self.logger.timing_since('HEAD.timing', start_time)
         return HTTPNoContent(request=req, headers=headers)
 
     def GET(self, req):
         """Handle HTTP GET request."""
+        start_time = time.time()
         try:
             drive, part, account, container, obj = split_path(
                 unquote(req.path), 4, 5, True)
         except ValueError, err:
+            self.logger.increment('GET.errors')
             return HTTPBadRequest(body=str(err), content_type='text/plain',
                                 request=req)
         if self.mount_check and not check_mount(self.root, drive):
-            return Response(status='507 %s is not mounted' % drive)
+            self.logger.increment('GET.errors')
+            return HTTPInsufficientStorage(drive=drive, request=req)
         broker = self._get_container_broker(drive, part, account, container)
         broker.pending_timeout = 0.1
         broker.stale_reads_ok = True
         if broker.is_deleted():
+            self.logger.timing_since('GET.timing', start_time)
             return HTTPNotFound(request=req)
         info = broker.get_info()
         resp_headers = {
@@ -302,6 +336,7 @@ class ContainerController(object):
                         body='Maximum limit is %d' % CONTAINER_LISTING_LIMIT)
             query_format = get_param(req, 'format')
         except UnicodeDecodeError, err:
+            self.logger.increment('GET.errors')
             return HTTPBadRequest(body='parameters not utf8',
                                   content_type='text/plain', request=req)
         if query_format:
@@ -312,6 +347,7 @@ class ContainerController(object):
                                      'application/xml', 'text/xml'],
                                     default_match='text/plain')
         except AssertionError, err:
+            self.logger.increment('GET.errors')
             return HTTPBadRequest(body='bad accept header: %s' % req.accept,
                                   content_type='text/plain', request=req)
         container_list = broker.list_objects_iter(limit, marker, end_marker,
@@ -364,53 +400,66 @@ class ContainerController(object):
                 ''.join(xml_output), '</container>'])
         else:
             if not container_list:
+                self.logger.timing_since('GET.timing', start_time)
                 return HTTPNoContent(request=req, headers=resp_headers)
             container_list = '\n'.join(r[0] for r in container_list) + '\n'
         ret = Response(body=container_list, request=req, headers=resp_headers)
         ret.content_type = out_content_type
         ret.charset = 'utf-8'
+        self.logger.timing_since('GET.timing', start_time)
         return ret
 
     def REPLICATE(self, req):
         """
         Handle HTTP REPLICATE request (json-encoded RPC calls for replication.)
         """
+        start_time = time.time()
         try:
             post_args = split_path(unquote(req.path), 3)
         except ValueError, err:
+            self.logger.increment('REPLICATE.errors')
             return HTTPBadRequest(body=str(err), content_type='text/plain',
                                 request=req)
         drive, partition, hash = post_args
         if self.mount_check and not check_mount(self.root, drive):
-            return Response(status='507 %s is not mounted' % drive)
+            self.logger.increment('REPLICATE.errors')
+            return HTTPInsufficientStorage(drive=drive, request=req)
         try:
             args = simplejson.load(req.environ['wsgi.input'])
         except ValueError, err:
+            self.logger.increment('REPLICATE.errors')
             return HTTPBadRequest(body=str(err), content_type='text/plain')
         ret = self.replicator_rpc.dispatch(post_args, args)
         ret.request = req
+        self.logger.timing_since('REPLICATE.timing', start_time)
         return ret
 
     def POST(self, req):
         """Handle HTTP POST request."""
+        start_time = time.time()
         try:
             drive, part, account, container = split_path(unquote(req.path), 4)
         except ValueError, err:
+            self.logger.increment('POST.errors')
             return HTTPBadRequest(body=str(err), content_type='text/plain',
                                   request=req)
         if 'x-timestamp' not in req.headers or \
                 not check_float(req.headers['x-timestamp']):
+            self.logger.increment('POST.errors')
             return HTTPBadRequest(body='Missing or bad timestamp',
                 request=req, content_type='text/plain')
         if 'x-container-sync-to' in req.headers:
             err = validate_sync_to(req.headers['x-container-sync-to'],
                                    self.allowed_sync_hosts)
             if err:
+                self.logger.increment('POST.errors')
                 return HTTPBadRequest(err)
         if self.mount_check and not check_mount(self.root, drive):
-            return Response(status='507 %s is not mounted' % drive)
+            self.logger.increment('POST.errors')
+            return HTTPInsufficientStorage(drive=drive, request=req)
         broker = self._get_container_broker(drive, part, account, container)
         if broker.is_deleted():
+            self.logger.timing_since('POST.timing', start_time)
             return HTTPNotFound(request=req)
         timestamp = normalize_timestamp(req.headers['x-timestamp'])
         metadata = {}
@@ -425,6 +474,7 @@ class ContainerController(object):
                         broker.metadata['X-Container-Sync-To'][0]:
                     broker.set_x_container_sync_points(-1, -1)
             broker.update_metadata(metadata)
+        self.logger.timing_since('POST.timing', start_time)
         return HTTPNoContent(request=req)
 
     def __call__(self, env, start_response):
