@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2011 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,13 +13,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import os
 from time import ctime, time
 from random import random, shuffle
 from struct import unpack_from
 
 from eventlet import sleep, Timeout
 
+import swift.common.db
 from swift.container import server as container_server
 from swift.common.client import ClientException, delete_object, put_object, \
     quote
@@ -27,7 +27,7 @@ from swift.common.direct_client import direct_get_object
 from swift.common.ring import Ring
 from swift.common.db import ContainerBroker
 from swift.common.utils import audit_location_generator, get_logger, \
-    hash_path, normalize_timestamp, TRUE_VALUES, validate_sync_to, whataremyips
+    hash_path, TRUE_VALUES, validate_sync_to, whataremyips
 from swift.common.daemon import Daemon
 
 
@@ -172,13 +172,14 @@ class ContainerSync(Daemon):
         self.reported = time()
         swift_dir = conf.get('swift_dir', '/etc/swift')
         #: swift.common.ring.Ring for locating containers.
-        self.container_ring = container_ring or \
-            Ring(os.path.join(swift_dir, 'container.ring.gz'))
+        self.container_ring = container_ring or Ring(swift_dir,
+                ring_name='container')
         #: swift.common.ring.Ring for locating objects.
-        self.object_ring = object_ring or \
-            Ring(os.path.join(swift_dir, 'object.ring.gz'))
+        self.object_ring = object_ring or Ring(swift_dir, ring_name='object')
         self._myips = whataremyips()
         self._myport = int(conf.get('bind_port', 6001))
+        swift.common.db.DB_PREALLOCATION = \
+            conf.get('db_preallocation', 't').lower() in TRUE_VALUES
 
     def run_forever(self):
         """
