@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2011 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,16 +15,11 @@
 
 import os
 import time
-import uuid
-import errno
-from hashlib import md5
-from random import random
 
 from eventlet import Timeout
 
 from swift.obj import server as object_server
-from swift.obj.replicator import invalidate_hash
-from swift.common.utils import get_logger, renamer, audit_location_generator, \
+from swift.common.utils import get_logger, audit_location_generator, \
     ratelimit_sleep, TRUE_VALUES
 from swift.common.exceptions import AuditException, DiskFileError, \
     DiskFileNotExist
