@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2011 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -36,6 +36,7 @@ from swift.common.utils import whataremyips, unlink_older_than, lock_path, \
         TRUE_VALUES
 from swift.common.bufferedhttp import http_connect
 from swift.common.daemon import Daemon
+from swift.common.http import HTTP_OK, HTTP_INSUFFICIENT_STORAGE
 
 hubs.use_hub('poll')
 
@@ -236,7 +237,7 @@ class ObjectReplicator(Daemon):
         self.port = int(conf.get('bind_port', 6000))
         self.concurrency = int(conf.get('concurrency', 1))
         self.stats_interval = int(conf.get('stats_interval', '300'))
-        self.object_ring = Ring(join(self.swift_dir, 'object.ring.gz'))
+        self.object_ring = Ring(self.swift_dir, ring_name='object')
         self.ring_check_interval = int(conf.get('ring_check_interval', 15))
         self.next_check = time.time() + self.ring_check_interval
         self.reclaim_age = int(conf.get('reclaim_age', 86400 * 7))
@@ -408,12 +409,12 @@ class ObjectReplicator(Daemon):
                         resp = http_connect(node['ip'], node['port'],
                                 node['device'], job['partition'], 'REPLICATE',
                             '', headers={'Content-Length': '0'}).getresponse()
-                        if resp.status == 507:
+                        if resp.status == HTTP_INSUFFICIENT_STORAGE:
                             self.logger.error(_('%(ip)s/%(device)s responded'
                                     ' as unmounted'), node)
                             attempts_left += 1
                             continue
-                        if resp.status != 200:
+                        if resp.status != HTTP_OK:
                             self.logger.error(_("Invalid response %(resp)s "
                                 "from %(ip)s"),
                                 {'resp': resp.status, 'ip': node['ip']})
