@@ -39,9 +39,8 @@ class ObjectUpdater(Daemon):
         self.devices = conf.get('devices', '/srv/node')
         self.mount_check = conf.get('mount_check', 'true').lower() in \
                               ('true', 't', '1', 'on', 'yes', 'y')
-        swift_dir = conf.get('swift_dir', '/etc/swift')
+        self.swift_dir = conf.get('swift_dir', '/etc/swift')
         self.interval = int(conf.get('interval', 300))
-        self.container_ring_path = os.path.join(swift_dir, 'container.ring.gz')
         self.container_ring = None
         self.concurrency = int(conf.get('concurrency', 1))
         self.slowdown = float(conf.get('slowdown', 0.01))
@@ -53,9 +52,7 @@ class ObjectUpdater(Daemon):
     def get_container_ring(self):
         """Get the container ring.  Load it, if it hasn't been yet."""
         if not self.container_ring:
-            self.logger.debug(
-                _('Loading container ring from %s'), self.container_ring_path)
-            self.container_ring = Ring(self.container_ring_path)
+            self.container_ring = Ring(self.swift_dir, ring_name='container')
         return self.container_ring
 
     def run_forever(self, *args, **kwargs):
@@ -176,6 +173,7 @@ class ObjectUpdater(Daemon):
         obj = '/%s/%s/%s' % \
               (update['account'], update['container'], update['obj'])
         success = True
+        new_successes = False
         for node in nodes:
             if node['id'] not in successes:
                 status = self.object_update(node, part, update['op'], obj,
@@ -184,6 +182,7 @@ class ObjectUpdater(Daemon):
                     success = False
                 else:
                     successes.append(node['id'])
+                    new_successes = True
         if success:
             self.successes += 1
             self.logger.debug(_('Update sent for %(obj)s %(path)s'),
@@ -193,8 +192,9 @@ class ObjectUpdater(Daemon):
             self.failures += 1
             self.logger.debug(_('Update failed for %(obj)s %(path)s'),
                 {'obj': obj, 'path': update_path})
-            update['successes'] = successes
-            write_pickle(update, update_path, os.path.join(device, 'tmp'))
+            if new_successes:
+                update['successes'] = successes
+                write_pickle(update, update_path, os.path.join(device, 'tmp'))
 
     def object_update(self, node, part, op, obj, headers):
         """
