@@ -11,7 +11,7 @@ from test import get_config
 
 from swift.common.client import get_auth, http_connection
 
-conf = get_config()
+conf = get_config('func_test')
 
 # If no conf was read, we will fall back to old school env vars
 swift_test_auth = os.environ.get('SWIFT_TEST_AUTH')
