@@ -56,15 +56,21 @@ class FakeMemcache(object):
 
 class FakeApp(object):
 
-    def __init__(self, status_headers_body_iter=None):
+    def __init__(self, status_headers_body_iter=None, acl=None, sync_key=None):
         self.calls = 0
         self.status_headers_body_iter = status_headers_body_iter
         if not self.status_headers_body_iter:
             self.status_headers_body_iter = iter([('404 Not Found', {}, '')])
+        self.acl = acl
+        self.sync_key = sync_key
 
     def __call__(self, env, start_response):
         self.calls += 1
         self.request = Request.blank('', environ=env)
+        if self.acl:
+            self.request.acl = self.acl
+        if self.sync_key:
+            self.request.environ['swift_sync_key'] = self.sync_key
         if 'swift.authorize' in env:
             resp = env['swift.authorize'](self.request)
             if resp:
@@ -133,16 +139,45 @@ class TestAuth(unittest.TestCase):
         ath = auth.filter_factory({'auth_prefix': 'test'})(app)
         self.assertEquals(ath.auth_prefix, '/test/')
 
-    def test_top_level_ignore(self):
+    def test_top_level_deny(self):
         resp = self._make_request('/').get_response(self.test_auth)
-        self.assertEquals(resp.status_int, 404)
+        self.assertEquals(resp.status_int, 401)
+        self.assertEquals(resp.environ['swift.authorize'],
+                          self.test_auth.denied_response)
 
     def test_anon(self):
-        resp = self._make_request('/v1/AUTH_account').get_response(self.test_auth)
+        resp = \
+            self._make_request('/v1/AUTH_account').get_response(self.test_auth)
         self.assertEquals(resp.status_int, 401)
         self.assertEquals(resp.environ['swift.authorize'],
                           self.test_auth.authorize)
 
+    def test_override_asked_for_but_not_allowed(self):
+        self.test_auth = \
+            auth.filter_factory({'allow_overrides': 'false'})(FakeApp())
+        req = self._make_request('/v1/AUTH_account',
+                                 environ={'swift.authorize_override': True})
+        resp = req.get_response(self.test_auth)
+        self.assertEquals(resp.status_int, 401)
+        self.assertEquals(resp.environ['swift.authorize'],
+                          self.test_auth.authorize)
+
+    def test_override_asked_for_and_allowed(self):
+        self.test_auth = \
+            auth.filter_factory({'allow_overrides': 'true'})(FakeApp())
+        req = self._make_request('/v1/AUTH_account',
+                                 environ={'swift.authorize_override': True})
+        resp = req.get_response(self.test_auth)
+        self.assertEquals(resp.status_int, 404)
+        self.assertTrue('swift.authorize' not in resp.environ)
+
+    def test_override_default_allowed(self):
+        req = self._make_request('/v1/AUTH_account',
+                                 environ={'swift.authorize_override': True})
+        resp = req.get_response(self.test_auth)
+        self.assertEquals(resp.status_int, 404)
+        self.assertTrue('swift.authorize' not in resp.environ)
+
     def test_auth_deny_non_reseller_prefix(self):
         resp = self._make_request('/v1/BLAH_account',
             headers={'X-Auth-Token': 'BLAH_t'}).get_response(self.test_auth)
@@ -289,30 +324,35 @@ class TestAuth(unittest.TestCase):
         self.assertEquals(self.test_auth.authorize(req), None)
 
     def test_account_put_permissions(self):
-        req = self._make_request('/v1/AUTH_new', environ={'REQUEST_METHOD': 'PUT'})
+        req = self._make_request('/v1/AUTH_new',
+                environ={'REQUEST_METHOD': 'PUT'})
         req.remote_user = 'act:usr,act'
         resp = self.test_auth.authorize(req)
         self.assertEquals(resp.status_int, 403)
 
-        req = self._make_request('/v1/AUTH_new', environ={'REQUEST_METHOD': 'PUT'})
+        req = self._make_request('/v1/AUTH_new',
+                environ={'REQUEST_METHOD': 'PUT'})
         req.remote_user = 'act:usr,act,AUTH_other'
         resp = self.test_auth.authorize(req)
         self.assertEquals(resp.status_int, 403)
 
         # Even PUTs to your own account as account admin should fail
-        req = self._make_request('/v1/AUTH_old', environ={'REQUEST_METHOD': 'PUT'})
+        req = self._make_request('/v1/AUTH_old',
+                environ={'REQUEST_METHOD': 'PUT'})
         req.remote_user = 'act:usr,act,AUTH_old'
         resp = self.test_auth.authorize(req)
         self.assertEquals(resp.status_int, 403)
 
-        req = self._make_request('/v1/AUTH_new', environ={'REQUEST_METHOD': 'PUT'})
+        req = self._make_request('/v1/AUTH_new',
+                environ={'REQUEST_METHOD': 'PUT'})
         req.remote_user = 'act:usr,act,.reseller_admin'
         resp = self.test_auth.authorize(req)
         self.assertEquals(resp, None)
 
         # .super_admin is not something the middleware should ever see or care
         # about
-        req = self._make_request('/v1/AUTH_new', environ={'REQUEST_METHOD': 'PUT'})
+        req = self._make_request('/v1/AUTH_new',
+                environ={'REQUEST_METHOD': 'PUT'})
         req.remote_user = 'act:usr,act,.super_admin'
         resp = self.test_auth.authorize(req)
         self.assertEquals(resp.status_int, 403)
@@ -383,6 +423,152 @@ class TestAuth(unittest.TestCase):
             headers={'X-Auth-User': 'act:usr'}).get_response(self.test_auth)
         self.assertEquals(resp.status_int, 401)
 
+    def test_allowed_sync_hosts(self):
+        a = auth.filter_factory({'super_admin_key': 'supertest'})(FakeApp())
+        self.assertEquals(a.allowed_sync_hosts, ['127.0.0.1'])
+        a = auth.filter_factory({'super_admin_key': 'supertest',
+            'allowed_sync_hosts':
+                '1.1.1.1,2.1.1.1, 3.1.1.1 , 4.1.1.1,, , 5.1.1.1'})(FakeApp())
+        self.assertEquals(a.allowed_sync_hosts,
+            ['1.1.1.1', '2.1.1.1', '3.1.1.1', '4.1.1.1', '5.1.1.1'])
+
+    def test_reseller_admin_is_owner(self):
+        orig_authorize = self.test_auth.authorize
+        owner_values = []
+
+        def mitm_authorize(req):
+            rv = orig_authorize(req)
+            owner_values.append(req.environ.get('swift_owner', False))
+            return rv
+
+        self.test_auth.authorize = mitm_authorize
+
+        req = self._make_request('/v1/AUTH_cfa',
+                headers={'X-Auth-Token': 'AUTH_t'})
+        req.remote_user = '.reseller_admin'
+        self.test_auth.authorize(req)
+        self.assertEquals(owner_values, [True])
+
+    def test_admin_is_owner(self):
+        orig_authorize = self.test_auth.authorize
+        owner_values = []
+
+        def mitm_authorize(req):
+            rv = orig_authorize(req)
+            owner_values.append(req.environ.get('swift_owner', False))
+            return rv
+
+        self.test_auth.authorize = mitm_authorize
+
+        req = self._make_request('/v1/AUTH_cfa',
+                headers={'X-Auth-Token': 'AUTH_t'})
+        req.remote_user = 'AUTH_cfa'
+        self.test_auth.authorize(req)
+        self.assertEquals(owner_values, [True])
+
+    def test_regular_is_not_owner(self):
+        orig_authorize = self.test_auth.authorize
+        owner_values = []
+
+        def mitm_authorize(req):
+            rv = orig_authorize(req)
+            owner_values.append(req.environ.get('swift_owner', False))
+            return rv
+
+        self.test_auth.authorize = mitm_authorize
+
+        req = self._make_request('/v1/AUTH_cfa/c',
+                            headers={'X-Auth-Token': 'AUTH_t'})
+        req.remote_user = 'act:usr'
+        self.test_auth.authorize(req)
+        self.assertEquals(owner_values, [False])
+
+    def test_sync_request_success(self):
+        self.test_auth.app = FakeApp(iter([('204 No Content', {}, '')]),
+                                     sync_key='secret')
+        req = self._make_request('/v1/AUTH_cfa/c/o',
+            environ={'REQUEST_METHOD': 'DELETE'},
+            headers={'x-container-sync-key': 'secret',
+                     'x-timestamp': '123.456'})
+        req.remote_addr = '127.0.0.1'
+        resp = req.get_response(self.test_auth)
+        self.assertEquals(resp.status_int, 204)
+
+    def test_sync_request_fail_key(self):
+        self.test_auth.app = FakeApp(iter([('204 No Content', {}, '')]),
+                                     sync_key='secret')
+        req = self._make_request('/v1/AUTH_cfa/c/o',
+            environ={'REQUEST_METHOD': 'DELETE'},
+            headers={'x-container-sync-key': 'wrongsecret',
+                     'x-timestamp': '123.456'})
+        req.remote_addr = '127.0.0.1'
+        resp = req.get_response(self.test_auth)
+        self.assertEquals(resp.status_int, 401)
+
+        self.test_auth.app = FakeApp(iter([('204 No Content', {}, '')]),
+                                     sync_key='othersecret')
+        req = self._make_request('/v1/AUTH_cfa/c/o',
+            environ={'REQUEST_METHOD': 'DELETE'},
+            headers={'x-container-sync-key': 'secret',
+                     'x-timestamp': '123.456'})
+        req.remote_addr = '127.0.0.1'
+        resp = req.get_response(self.test_auth)
+        self.assertEquals(resp.status_int, 401)
+
+        self.test_auth.app = FakeApp(iter([('204 No Content', {}, '')]),
+                                     sync_key=None)
+        req = self._make_request('/v1/AUTH_cfa/c/o',
+            environ={'REQUEST_METHOD': 'DELETE'},
+            headers={'x-container-sync-key': 'secret',
+                     'x-timestamp': '123.456'})
+        req.remote_addr = '127.0.0.1'
+        resp = req.get_response(self.test_auth)
+        self.assertEquals(resp.status_int, 401)
+
+    def test_sync_request_fail_no_timestamp(self):
+        self.test_auth.app = FakeApp(iter([('204 No Content', {}, '')]),
+                                     sync_key='secret')
+        req = self._make_request('/v1/AUTH_cfa/c/o',
+            environ={'REQUEST_METHOD': 'DELETE'},
+            headers={'x-container-sync-key': 'secret'})
+        req.remote_addr = '127.0.0.1'
+        resp = req.get_response(self.test_auth)
+        self.assertEquals(resp.status_int, 401)
+
+    def test_sync_request_fail_sync_host(self):
+        self.test_auth.app = FakeApp(iter([('204 No Content', {}, '')]),
+                                     sync_key='secret')
+        req = self._make_request('/v1/AUTH_cfa/c/o',
+            environ={'REQUEST_METHOD': 'DELETE'},
+            headers={'x-container-sync-key': 'secret',
+                     'x-timestamp': '123.456'})
+        req.remote_addr = '127.0.0.2'
+        resp = req.get_response(self.test_auth)
+        self.assertEquals(resp.status_int, 401)
+
+    def test_sync_request_success_lb_sync_host(self):
+        self.test_auth.app = FakeApp(iter([('204 No Content', {}, '')]),
+                                     sync_key='secret')
+        req = self._make_request('/v1/AUTH_cfa/c/o',
+            environ={'REQUEST_METHOD': 'DELETE'},
+            headers={'x-container-sync-key': 'secret',
+                     'x-timestamp': '123.456',
+                     'x-forwarded-for': '127.0.0.1'})
+        req.remote_addr = '127.0.0.2'
+        resp = req.get_response(self.test_auth)
+        self.assertEquals(resp.status_int, 204)
+
+        self.test_auth.app = FakeApp(iter([('204 No Content', {}, '')]),
+                                     sync_key='secret')
+        req = self._make_request('/v1/AUTH_cfa/c/o',
+            environ={'REQUEST_METHOD': 'DELETE'},
+            headers={'x-container-sync-key': 'secret',
+                     'x-timestamp': '123.456',
+                     'x-cluster-client-ip': '127.0.0.1'})
+        req.remote_addr = '127.0.0.2'
+        resp = req.get_response(self.test_auth)
+        self.assertEquals(resp.status_int, 204)
+
 
 class TestParseUserCreation(unittest.TestCase):
     def test_parse_user_creation(self):
