@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2011 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,7 +19,7 @@ from __future__ import with_statement
 import hashlib
 import os
 import unittest
-from shutil import rmtree
+from shutil import rmtree, copy
 from StringIO import StringIO
 from time import sleep, time
 from uuid import uuid4
@@ -27,6 +27,7 @@ from uuid import uuid4
 import simplejson
 import sqlite3
 
+import swift.common.db
 from swift.common.db import AccountBroker, chexor, ContainerBroker, \
     DatabaseBroker, DatabaseConnectionError, dict_factory, get_db_connection
 from swift.common.utils import normalize_timestamp
@@ -96,6 +97,13 @@ class TestDatabaseBroker(unittest.TestCase):
         rmtree(self.testdir, ignore_errors=1)
         os.mkdir(self.testdir)
 
+    def test_DB_PREALLOCATION_setting(self):
+        u = uuid4().hex
+        b = DatabaseBroker(u)
+        self.assertRaises(OSError, b._preallocate)
+        swift.common.db.DB_PREALLOCATION = False
+        b._preallocate()
+
     def tearDown(self):
         rmtree(self.testdir, ignore_errors=1)
 
@@ -140,24 +148,39 @@ class TestDatabaseBroker(unittest.TestCase):
             conn.execute('SELECT * FROM incoming_sync')
 
     def test_delete_db(self):
+        def init_stub(conn, put_timestamp):
+            conn.execute('CREATE TABLE test (one TEXT)')
+            conn.execute('CREATE TABLE test_stat (id TEXT)')
+            conn.execute('INSERT INTO test_stat (id) VALUES (?)',
+                        (str(uuid4),))
+            conn.execute('INSERT INTO test (one) VALUES ("1")')
+            conn.commit()
         stub_called = [False]
-        def stub(*args, **kwargs):
+        def delete_stub(*a, **kw):
             stub_called[0] = True
         broker = DatabaseBroker(':memory:')
-        broker._initialize = stub
+        broker.db_type = 'test'
+        broker._initialize = init_stub
+        # Initializes a good broker for us
         broker.initialize(normalize_timestamp('1'))
         self.assert_(broker.conn is not None)
-        broker._delete_db = stub
+        broker._delete_db = delete_stub
         stub_called[0] = False
         broker.delete_db('2')
         self.assert_(stub_called[0])
         broker = DatabaseBroker(os.path.join(self.testdir, '1.db'))
-        broker._initialize = stub
+        broker.db_type = 'test'
+        broker._initialize = init_stub
         broker.initialize(normalize_timestamp('1'))
-        broker._delete_db = stub
+        broker._delete_db = delete_stub
         stub_called[0] = False
         broker.delete_db('2')
         self.assert_(stub_called[0])
+        # ensure that metadata was cleared
+        m2 = broker.metadata
+        self.assert_(not any(v[0] for v in m2.itervalues()))
+        self.assert_(all(v[1] == normalize_timestamp('2')
+                        for v in m2.itervalues()))
 
     def test_get(self):
         broker = DatabaseBroker(':memory:')
@@ -199,6 +222,47 @@ class TestDatabaseBroker(unittest.TestCase):
         with broker.get() as conn:
             self.assertEquals(
                 [r[0] for r in conn.execute('SELECT * FROM test')], ['1'])
+        orig_renamer = swift.common.db.renamer
+        try:
+            swift.common.db.renamer = lambda a, b: b
+            qpath = os.path.dirname(os.path.dirname(os.path.dirname(
+                os.path.dirname(self.testdir))))
+            if qpath:
+                qpath += '/quarantined/tests/db'
+            else:
+                qpath = 'quarantined/tests/db'
+            # Test malformed database
+            copy(os.path.join(os.path.dirname(__file__),
+                              'malformed_example.db'),
+                 os.path.join(self.testdir, '1.db'))
+            broker = DatabaseBroker(os.path.join(self.testdir, '1.db'))
+            broker.db_type = 'test'
+            exc = None
+            try:
+                with broker.get() as conn:
+                    conn.execute('SELECT * FROM test')
+            except Exception, err:
+                exc = err
+            self.assertEquals(str(exc),
+                'Quarantined %s to %s due to malformed database' %
+                (self.testdir, qpath))
+            # Test corrupted database
+            copy(os.path.join(os.path.dirname(__file__),
+                              'corrupted_example.db'),
+                 os.path.join(self.testdir, '1.db'))
+            broker = DatabaseBroker(os.path.join(self.testdir, '1.db'))
+            broker.db_type = 'test'
+            exc = None
+            try:
+                with broker.get() as conn:
+                    conn.execute('SELECT * FROM test')
+            except Exception, err:
+                exc = err
+            self.assertEquals(str(exc),
+                'Quarantined %s to %s due to corrupted database' %
+                (self.testdir, qpath))
+        finally:
+            swift.common.db.renamer = orig_renamer
 
     def test_lock(self):
         broker = DatabaseBroker(os.path.join(self.testdir, '1.db'), timeout=.1)
@@ -550,7 +614,7 @@ class TestContainerBroker(unittest.TestCase):
                 raise Exception('OMG')
         except Exception:
             pass
-        self.assert_(broker.conn == None)
+        self.assert_(broker.conn is None)
 
     def test_empty(self):
         """ Test swift.common.db.ContainerBroker.empty """
@@ -1478,7 +1542,7 @@ class TestAccountBroker(unittest.TestCase):
                 raise Exception('OMG')
         except Exception:
             pass
-        self.assert_(broker.conn == None)
+        self.assert_(broker.conn is None)
 
     def test_empty(self):
         """ Test swift.common.db.AccountBroker.empty """
@@ -1750,9 +1814,9 @@ class TestAccountBroker(unittest.TestCase):
         self.assertEquals(listing[-1][0], '0/0099')
 
         listing = broker.list_containers_iter(100, '', '0/0050', None, '')
-        self.assertEquals(len(listing), 51)
+        self.assertEquals(len(listing), 50)
         self.assertEquals(listing[0][0], '0/0000')
-        self.assertEquals(listing[-1][0], '0/0050')
+        self.assertEquals(listing[-1][0], '0/0049')
 
         listing = broker.list_containers_iter(100, '0/0099', None, None, '')
         self.assertEquals(len(listing), 100)
