@@ -26,6 +26,7 @@ from getpass import getuser
 from shutil import rmtree
 from StringIO import StringIO
 from collections import defaultdict
+from urllib import quote
 
 from eventlet import sleep
 from webob import Request
@@ -179,7 +180,7 @@ class TestWSGI(unittest.TestCase):
             @classmethod
             def fake_blank(cls, path, environ={}, body='', headers={}):
                 self.assertEquals(environ['swift.authorize']('test'), None)
-                self.assertEquals(environ['HTTP_X_TRANS_ID'], '1234')
+                self.assertFalse('HTTP_X_TRANS_ID' in environ)
         was_blank = Request.blank
         Request.blank = FakeReq.fake_blank
         wsgi.make_pre_authed_request({'HTTP_X_TRANS_ID': '1234'},
@@ -188,5 +189,11 @@ class TestWSGI(unittest.TestCase):
                                      'PUT', '/', headers={})
         Request.blank = was_blank
 
+    def test_pre_auth_req_with_quoted_path(self):
+        r = wsgi.make_pre_authed_request(
+            {'HTTP_X_TRANS_ID': '1234'}, 'PUT', path=quote('/a space'),
+            body='tester', headers={})
+        self.assertEquals(r.path, quote('/a space'))
+
 if __name__ == '__main__':
     unittest.main()
