@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2011 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -26,6 +26,7 @@ import time
 import tempfile
 from contextlib import contextmanager
 from eventlet.green import subprocess
+from eventlet import Timeout, tpool
 from test.unit import FakeLogger
 from swift.common import utils
 from swift.common.utils import hash_path, mkdirs, normalize_timestamp
@@ -125,7 +126,7 @@ def _create_test_ring(path):
     pickle.dump(ring.RingData(intended_replica2part2dev_id,
         intended_devs, intended_part_shift),
         GzipFile(testgz, 'wb'))
-    return ring.Ring(testgz, reload_time=intended_reload_time)
+    return ring.Ring(path, ring_name='object', reload_time=intended_reload_time)
 
 
 class TestObjectReplicator(unittest.TestCase):
@@ -201,10 +202,32 @@ class TestObjectReplicator(unittest.TestCase):
         hashed, hashes = object_replicator.get_hashes(part, do_listdir=True)
         self.assertEquals(hashed, 0)
         self.assert_('a83' in hashes)
-        hashed, hashes = object_replicator.get_hashes(part, recalculate=['a83'])
+        hashed, hashes = object_replicator.get_hashes(part,
+                                                      recalculate=['a83'])
         self.assertEquals(hashed, 1)
         self.assert_('a83' in hashes)
 
+    def test_hash_suffix_hash_dir_is_file_quarantine(self):
+        df = DiskFile(self.devices, 'sda', '0', 'a', 'c', 'o', FakeLogger())
+        mkdirs(os.path.dirname(df.datadir))
+        open(df.datadir, 'wb').close()
+        ohash = hash_path('a', 'c', 'o')
+        data_dir = ohash[-3:]
+        whole_path_from = os.path.join(self.objects, '0', data_dir)
+        orig_quarantine_renamer = object_replicator.quarantine_renamer
+        called = [False]
+
+        def wrapped(*args, **kwargs):
+            called[0] = True
+            return orig_quarantine_renamer(*args, **kwargs)
+
+        try:
+            object_replicator.quarantine_renamer = wrapped
+            object_replicator.hash_suffix(whole_path_from, 101)
+        finally:
+            object_replicator.quarantine_renamer = orig_quarantine_renamer
+        self.assertTrue(called[0])
+
     def test_hash_suffix_one_file(self):
         df = DiskFile(self.devices, 'sda', '0', 'a', 'c', 'o', FakeLogger())
         mkdirs(df.datadir)
@@ -343,42 +366,106 @@ class TestObjectReplicator(unittest.TestCase):
             dict(swift_dir=self.testdir, devices=self.devices,
                 mount_check='false', timeout='300', stats_interval='1'))
         was_connector = object_replicator.http_connect
-        object_replicator.http_connect = mock_http_connect(200)
-        # Write some files into '1' and run replicate- they should be moved
-        # to the other partitoins and then node should get deleted.
-        cur_part = '1'
-        df = DiskFile(self.devices, 'sda', cur_part, 'a', 'c', 'o',
-                      FakeLogger())
-        mkdirs(df.datadir)
-        f = open(os.path.join(df.datadir,
-                              normalize_timestamp(time.time()) + '.data'),
-                 'wb')
-        f.write('1234567890')
-        f.close()
-        ohash = hash_path('a', 'c', 'o')
-        data_dir = ohash[-3:]
-        whole_path_from = os.path.join(self.objects, cur_part, data_dir)
-        process_arg_checker = []
-        nodes = [node for node in
-                 self.ring.get_part_nodes(int(cur_part)) \
-                     if node['ip'] not in _ips()]
-        for node in nodes:
-            rsync_mod = '[%s]::object/sda/objects/%s' % (node['ip'], cur_part)
-            process_arg_checker.append(
-                (0, '', ['rsync', whole_path_from, rsync_mod]))
-        self.assertTrue(os.access(os.path.join(self.objects,
-                                               '1', data_dir, ohash),
-                                  os.F_OK))
-        with _mock_process(process_arg_checker):
-            replicator.run_once()
-        self.assertFalse(process_errors)
-        for i, result in [('0', True), ('1', False),
-                          ('2', True), ('3', True)]:
-            self.assertEquals(os.access(
-                    os.path.join(self.objects,
-                                 i, object_replicator.HASH_FILE),
-                    os.F_OK), result)
-        object_replicator.http_connect = was_connector
+        try:
+            object_replicator.http_connect = mock_http_connect(200)
+            # Write some files into '1' and run replicate- they should be moved
+            # to the other partitoins and then node should get deleted.
+            cur_part = '1'
+            df = DiskFile(self.devices, 'sda', cur_part, 'a', 'c', 'o',
+                          FakeLogger())
+            mkdirs(df.datadir)
+            f = open(os.path.join(df.datadir,
+                                  normalize_timestamp(time.time()) + '.data'),
+                     'wb')
+            f.write('1234567890')
+            f.close()
+            ohash = hash_path('a', 'c', 'o')
+            data_dir = ohash[-3:]
+            whole_path_from = os.path.join(self.objects, cur_part, data_dir)
+            process_arg_checker = []
+            nodes = [node for node in
+                     self.ring.get_part_nodes(int(cur_part)) \
+                         if node['ip'] not in _ips()]
+            for node in nodes:
+                rsync_mod = '[%s]::object/sda/objects/%s' % (node['ip'],
+                                                             cur_part)
+                process_arg_checker.append(
+                    (0, '', ['rsync', whole_path_from, rsync_mod]))
+            self.assertTrue(os.access(os.path.join(self.objects,
+                                                   '1', data_dir, ohash),
+                                      os.F_OK))
+            with _mock_process(process_arg_checker):
+                replicator.run_once()
+            self.assertFalse(process_errors)
+            for i, result in [('0', True), ('1', False),
+                              ('2', True), ('3', True)]:
+                self.assertEquals(os.access(
+                        os.path.join(self.objects,
+                                     i, object_replicator.HASH_FILE),
+                        os.F_OK), result)
+        finally:
+            object_replicator.http_connect = was_connector
+
+    def test_run_once_recover_from_timeout(self):
+        replicator = object_replicator.ObjectReplicator(
+            dict(swift_dir=self.testdir, devices=self.devices,
+                mount_check='false', timeout='300', stats_interval='1'))
+        was_connector = object_replicator.http_connect
+        was_get_hashes = object_replicator.get_hashes
+        was_execute = tpool.execute
+        self.get_hash_count = 0
+        try:
+
+            def fake_get_hashes(*args, **kwargs):
+                self.get_hash_count += 1
+                if self.get_hash_count == 3:
+                    # raise timeout on last call to get hashes
+                    raise Timeout()
+                return 2, {'abc': 'def'}
+
+            def fake_exc(tester, *args, **kwargs):
+                if 'Error syncing partition' in args[0]:
+                    tester.i_failed = True
+
+            self.i_failed = False
+            object_replicator.http_connect = mock_http_connect(200)
+            object_replicator.get_hashes = fake_get_hashes
+            replicator.logger.exception = \
+                lambda *args, **kwargs: fake_exc(self, *args, **kwargs)
+            # Write some files into '1' and run replicate- they should be moved
+            # to the other partitions and then node should get deleted.
+            cur_part = '1'
+            df = DiskFile(self.devices, 'sda', cur_part, 'a', 'c', 'o',
+                          FakeLogger())
+            mkdirs(df.datadir)
+            f = open(os.path.join(df.datadir,
+                                  normalize_timestamp(time.time()) + '.data'),
+                     'wb')
+            f.write('1234567890')
+            f.close()
+            ohash = hash_path('a', 'c', 'o')
+            data_dir = ohash[-3:]
+            whole_path_from = os.path.join(self.objects, cur_part, data_dir)
+            process_arg_checker = []
+            nodes = [node for node in
+                     self.ring.get_part_nodes(int(cur_part)) \
+                         if node['ip'] not in _ips()]
+            for node in nodes:
+                rsync_mod = '[%s]::object/sda/objects/%s' % (node['ip'],
+                                                             cur_part)
+                process_arg_checker.append(
+                    (0, '', ['rsync', whole_path_from, rsync_mod]))
+            self.assertTrue(os.access(os.path.join(self.objects,
+                                                   '1', data_dir, ohash),
+                                      os.F_OK))
+            with _mock_process(process_arg_checker):
+                replicator.run_once()
+            self.assertFalse(process_errors)
+            self.assertFalse(self.i_failed)
+        finally:
+            object_replicator.http_connect = was_connector
+            object_replicator.get_hashes = was_get_hashes
+            tpool.execute = was_execute
 
     def test_run(self):
         with _mock_process([(0, '')] * 100):
